/* callbacks.c
 *
 * Copyright (C) 2000 Stefano Bellotti
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

/*
 * button click
 */
void
on_button_color_clicked                (GtkWidget *widget,
                                        gpointer color)
{
  int col, score, score_opponent, half;

  col = GPOINTER_TO_INT (color);

  /* enable old color */
  gtk_widget_set_sensitive (game.button_color[game_table_get_color (game.table, game.active_player)], 1);

  /* disable new color */
  gtk_widget_set_sensitive (game.button_color[col], 0);

  /* do action */
  score = game_table_play_color (game.table, col, game.active_player);

  /* refresh */
  game_update_backup_xpm ();
  game_refresh_screen ();

  /* write score */
  game_write_perc (game.active_player, score);

  /* check game over */
  score_opponent = game_table_get_number_of_rhombs (game.table, (game.active_player + 1) % 2);
  half = game.table->width * game.table->height / 2;
  if (score == half && score_opponent == half) game_end (2);
  else if (score > half) game_end (game.active_player);
  else
  {
    /* opponent turn */
    game.active_player = (game.active_player + 1) % 2;

    /* computer turn? */
    if (game.player_cpu[game.active_player]) game_computer_turn ();
  }
}

/*
 * Redraw hidden
 */
gint
on_drawingarea1_expose_event           (GtkWidget       *widget,
                                        GdkEventExpose  *event)
{
  gdk_draw_pixmap(widget->window,
                  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
                  game.backup_xpm,
                  event->area.x, event->area.y,
                  event->area.x, event->area.y,
                  event->area.width, event->area.height);

 return FALSE;
}

gint
on_app1_delete_event                   (GtkWidget       *widget,
                                        GdkEventAny     *event,
                                        gpointer         user_data)
{
  gtk_main_quit();
  return FALSE;
}


void
on_new_game1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  game_start_new ();
}


void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_main_quit();
}


void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget * mbox;

  mbox = gnome_message_box_new (_("Preferences window will be added in a future version :-)"),
                                GNOME_MESSAGE_BOX_INFO,
                                GNOME_STOCK_BUTTON_OK,
                                NULL);

  gnome_dialog_set_parent(GNOME_DIALOG (mbox), GTK_WINDOW (game.app1));

  gtk_window_set_modal (GTK_WINDOW (mbox), TRUE);

  gtk_widget_show (mbox);
}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *about1;

  about1 = create_about1 ();
  gtk_widget_show (about1);
}

