/* interface.c
 *
 * Copyright (C) 2000 Stefano Bellotti
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "defaultxpm.h"

static gchar* colorname[] =
{
 N_("Gray"),
 N_("Red"),
 N_("Green"),
 N_("Blue"),
 N_("Cyan"),
 N_("Magenta"),
 N_("Yellow")
};

static GnomeUIInfo game1_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_NEW_GAME_ITEM (on_new_game1_activate, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_EXIT_ITEM (on_exit1_activate, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo settings1_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_PREFERENCES_ITEM (on_preferences1_activate, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo help1_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_ABOUT_ITEM (on_about1_activate, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo menubar1_uiinfo[] =
{
  GNOMEUIINFO_MENU_GAME_TREE (game1_menu_uiinfo),
  GNOMEUIINFO_MENU_SETTINGS_TREE (settings1_menu_uiinfo),
  GNOMEUIINFO_MENU_HELP_TREE (help1_menu_uiinfo),
  GNOMEUIINFO_END
};

GtkWidget*
create_app1 (void)
{
  GtkWidget *app1;
  GtkWidget *dock1;
  GtkWidget *vbox1;
  GtkWidget *hbox1;
  //GtkWidget *label_score[2];
  GtkWidget *scrolledwindow1;
  GtkWidget *viewport1;
  //GtkWidget *drawingarea1;
  GtkWidget *hbox2;
  //GtkWidget *button_color[7];
  GtkWidget *vbox_color[7];
  GtkWidget *pixmap_color[7];
  GtkWidget *label_color[7];
  //GtkWidget *appbar1;
  //GdkPixmap *rhomb_xpm[7];
  GdkBitmap *rhomb_mask[7];
  int i;

  app1 = gnome_app_new (PACKAGE, _("Seven Colors"));
  gtk_object_set_data (GTK_OBJECT (app1), "app1", app1);
  gtk_window_set_policy (GTK_WINDOW (app1), TRUE, FALSE, FALSE);
  gtk_signal_connect(GTK_OBJECT(app1), "delete_event",
                     GTK_SIGNAL_FUNC(on_app1_delete_event), NULL);

  dock1 = GNOME_APP (app1)->dock;
  gtk_widget_ref (dock1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "dock1", dock1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dock1);

  gnome_app_create_menus (GNOME_APP (app1), menubar1_uiinfo);

  gtk_widget_ref (menubar1_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "game1",
                            menubar1_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (game1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "new_game1",
                            game1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (game1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "separatore1",
                            game1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (game1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "exit1",
                            game1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (menubar1_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "settings1",
                            menubar1_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (settings1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "preferences1",
                            settings1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (menubar1_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "help1",
                            menubar1_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (help1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "about1",
                            help1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gnome_app_set_contents (GNOME_APP (app1), vbox1);

  hbox1 = gtk_hbox_new (TRUE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox1), 5);

  /* labels for players score */
  for (i = 0; i < 2; i++)
  {
    game.label_score[i] = gtk_label_new ("");
    gtk_widget_ref (game.label_score[i]);
    gtk_object_set_data_full (GTK_OBJECT (app1), g_strdup_printf("label_score%d",i),
                              game.label_score[i], (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (game.label_score[i]);
    gtk_box_pack_start (GTK_BOX (hbox1), game.label_score[i], TRUE, TRUE, 0);
  }

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "scrolledwindow1", scrolledwindow1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);

  viewport1 = gtk_viewport_new (NULL, NULL);
  gtk_widget_ref (viewport1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "viewport1", viewport1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (viewport1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), viewport1);

  game.drawingarea1 = gtk_drawing_area_new ();
  gtk_widget_ref (game.drawingarea1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "drawingarea1", game.drawingarea1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (game.drawingarea1);
  gtk_container_add (GTK_CONTAINER (viewport1), game.drawingarea1);

  hbox2 = gtk_hbox_new (TRUE, 5);
  gtk_widget_ref (hbox2);
  gtk_object_set_data_full (GTK_OBJECT (app1), "hbox2", hbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox2, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox2), 5);

  for (i = 0; i < 7; i++)
  {
    /* button color */
    game.button_color[i] = gtk_button_new ();
    gtk_widget_ref (game.button_color[i]);
    gtk_object_set_data_full ( GTK_OBJECT (app1), g_strdup_printf ("button_color%d", i),
                              game.button_color[i], (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (game.button_color[i]);
    gtk_box_pack_start (GTK_BOX (hbox2), game.button_color[i], TRUE, TRUE, 0);

    /* event click */
    gtk_signal_connect( GTK_OBJECT (game.button_color[i]), "clicked",
                     GTK_SIGNAL_FUNC (on_button_color_clicked), GINT_TO_POINTER (i));


    /* vert. container */
    vbox_color[i] = gtk_vbox_new (FALSE, 0);
    gtk_widget_ref (vbox_color[i]);
    gtk_object_set_data_full (GTK_OBJECT (app1), g_strdup_printf("vbox_color%d",i),
                              vbox_color[i], (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (vbox_color[i]);
    gtk_container_add (GTK_CONTAINER (game.button_color[i]), vbox_color[i]);

    /* create xpm */
    game.rhomb_xpm[i]=gdk_pixmap_colormap_create_from_xpm_d(NULL, gtk_widget_get_colormap(app1),
                                                       &rhomb_mask[i],NULL,
                                                       (gchar **)rhomb_xpm_data[i]);
    pixmap_color[i]=gtk_pixmap_new(game.rhomb_xpm[i],rhomb_mask[i]);
    gtk_widget_ref (pixmap_color[i]);
    gtk_object_set_data_full (GTK_OBJECT (app1), g_strdup_printf("pixmap_color%d",i),
                              pixmap_color[i], (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (pixmap_color[i]);
    gtk_box_pack_start (GTK_BOX (vbox_color[i]), pixmap_color[i], FALSE, FALSE, 0);

    /* label color */
    label_color[i] = gtk_label_new (_(colorname[i]));
    gtk_widget_ref (label_color[i]);
    gtk_object_set_data_full (GTK_OBJECT (app1), g_strdup_printf("label_color%d",i),
                              label_color[i], (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (label_color[i]);
    gtk_box_pack_start (GTK_BOX (vbox_color[i]), label_color[i], FALSE, FALSE, 0);
  }

  /* get rhombs dimensions */
  gdk_window_get_size (game.rhomb_xpm[0], &game.rhomb_width, &game.rhomb_height);

  /* get total dimension */
  game.total_width = game.table->width * game.rhomb_width + game.rhomb_width / 2;
  game.total_height = game.table->height * game.rhomb_height / 2 + game.rhomb_height / 2;

  /* the backup pixmap */
  game.backup_xpm = gdk_pixmap_new (NULL, game.total_width, game.total_height,
                                    gtk_widget_get_visual(app1)->depth);
  for (i = 0; i < 7; i++)
  {
    /* Create GCs*/
    game.rhomb_gc[i] = gdk_gc_new (game.backup_xpm);
    gdk_gc_set_clip_mask (game.rhomb_gc[i], rhomb_mask[i]);
  }

  /* resize the drawing area */
  gtk_drawing_area_size (GTK_DRAWING_AREA (game.drawingarea1),
                        game.total_width, game.total_height);

  /* resize scrollbar */
  gtk_widget_set_usize (viewport1, game.total_width + 4, game.total_height + 4);


  /* refresh event */
  gtk_signal_connect (GTK_OBJECT (game.drawingarea1), "expose_event",
                      (GtkSignalFunc) on_drawingarea1_expose_event, NULL);

  game.appbar1 = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_NEVER);
  gtk_widget_ref (game.appbar1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "appbar1", game.appbar1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (game.appbar1);
  gnome_app_set_statusbar (GNOME_APP (app1), game.appbar1);

  gnome_app_install_menu_hints (GNOME_APP (app1), menubar1_uiinfo);

  return app1;
}

GtkWidget*
create_about1 (void)
{
  const gchar *authors[] = {
    "Stefano Bellotti <signo@lugbs.linux.it>",
    NULL
  };
  GtkWidget *about1;

  about1 = gnome_about_new (_("Seven Colors"), VERSION,
                        "Copyright (c) 2000 Stefano Bellotti",
                        authors,
                        _("A little addicting game, take over the gaming area with your color."),
                        NULL);
  gnome_dialog_set_parent(GNOME_DIALOG (about1), GTK_WINDOW (game.app1));
  gtk_object_set_data (GTK_OBJECT (about1), "about1", about1);
  gtk_window_set_modal (GTK_WINDOW (about1), TRUE);

  return about1;
}

/*
 * Redraw the backup xpm
 */
void game_update_backup_xpm (void)
{
  int i,j,col,x,y;

  /* clear the backup pixmap */
  gdk_draw_rectangle (game.backup_xpm, game.drawingarea1->style->black_gc,
                      TRUE, 0, 0, game.total_width, game.total_height);

  /* draw rhombs */
  for(i = 0; i < game.table->width ; i++)
    for(j = 0; j < game.table->height ; j++)
    {
      col = game.table->rhomb[i][j].col;
      x = i * game.rhomb_width + ( (j + 1) % 2) * game.rhomb_width / 2;
      y = j * game.rhomb_height / 2;

      gdk_gc_set_clip_origin (game.rhomb_gc[col], x, y);
      gdk_draw_pixmap (game.backup_xpm, game.rhomb_gc[col], game.rhomb_xpm[col],
                       0, 0, x, y, -1, -1);
    }
}

/*
 * Draw the backup xpm on the screen
 */
void game_refresh_screen (void)
{
  GdkRectangle update_rect;

  /* Define update rect */
  update_rect.x = 0;
  update_rect.y = 0;
  update_rect.width = game.total_width;
  update_rect.height = game.total_height;

  /* refresh screen */
  gtk_widget_draw (game.drawingarea1, &update_rect);
}

/*
 * Init a new game of the same size
 */
void game_start_new (void)
{
  int i;

  game.active_player = 0;

  /* random table */
  game_table_randomize (game.table);

  /* draw the table */
  game_update_backup_xpm ();

  /* refresh screen */
  game_refresh_screen ();

  /* enable all buttons */
  for (i = 0; i < 7; i++)
    gtk_widget_set_sensitive (game.button_color[i], 1);

  /* disable used buttons */
  gtk_widget_set_sensitive (game.button_color[game_table_get_color (game.table ,0)], 0);
  gtk_widget_set_sensitive (game.button_color[game_table_get_color (game.table ,1)], 0);

  /* write initial score */
  game_write_perc (0, game_table_get_number_of_rhombs(game.table, 0));
  game_write_perc (1, game_table_get_number_of_rhombs(game.table, 1));

  if (game.player_cpu[game.active_player]) game_computer_turn ();
}

/*
 * write player score
 */
void game_write_perc (int player, int rhombs_number)
{
  gchar *str;
  float perc;

  perc = rhombs_number * 100.0 / (game.table->width * game.table->height);
  str = g_strdup_printf ("%03.1f%%", perc);
  gtk_label_set_text ((GtkLabel *) game.label_score[player], str);
}

/*
 * End of the game
 */
void game_end (int winner)
{
  GtkWidget * mbox;
  gchar *msg;
  int i;

  /* disable all buttons */
  for (i = 0; i < 7; i++)
    gtk_widget_set_sensitive (game.button_color[i], 0);

  if (winner == 0 || winner == 1)
    msg = g_strdup_printf (_("Player %d wins!"), winner + 1);
  else
    msg = g_strdup_printf (_("Draw!"));

  mbox = gnome_message_box_new (msg,
                                GNOME_MESSAGE_BOX_INFO,
                                GNOME_STOCK_BUTTON_OK,
                                NULL);

  gnome_dialog_set_parent(GNOME_DIALOG (mbox), GTK_WINDOW (game.app1));

  gtk_window_set_modal (GTK_WINDOW (mbox), TRUE);

  gtk_widget_show (mbox);

  g_free (msg);
}

/*
 * Computer turn
 */
void game_computer_turn (void)
{
  int col, score, score_opponent, half;

  /* enable old color */
  gtk_widget_set_sensitive (game.button_color[game_table_get_color (game.table, game.active_player)], 1);

  /* do action */
  score = game_table_computer_play (game.table, game.active_player, &col);

  /* disable new color */
  gtk_widget_set_sensitive (game.button_color[col], 0);

  /* refresh */
  game_update_backup_xpm ();
  game_refresh_screen ();

  /* write score */
  game_write_perc (game.active_player, score);

  /* check game over */
  score_opponent = game_table_get_number_of_rhombs (game.table, (game.active_player + 1) % 2);
  half = game.table->width * game.table->height / 2;
  if (score == half && score_opponent == half) game_end (2);
  else if (score > half) game_end (game.active_player);
  else
  {
    /* opponent turn */
    game.active_player = (game.active_player + 1) % 2;

    /* computer turn? */
    if (game.player_cpu[game.active_player]) game_computer_turn ();
  }
}