/* main.c
 *
 * Copyright (C) 2000 Stefano Bellotti
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "interface.h"
#include "support.h"

static int width = 18, height = 18;
static char *p1 = NULL, *p2 = NULL;

struct poptOption options[] = {
  {
    "width",
    'w',
    POPT_ARG_INT,
    &width,
    0,
    N_("Width of the game table"),
    N_("WIDTH")
  },
  {
    "height",
    'h',
    POPT_ARG_INT,
    &height,
    0,
    N_("Height of the game table"),
    N_("HEIGHT")
  },
  {
    "player1",
    '1',
    POPT_ARG_STRING,
    &p1,
    0,
    N_("Player 1 type ('c' for computer, 'h' for human)"),
    N_("TYPE")
  },
  {
    "player2",
    '2',
    POPT_ARG_STRING,
    &p2,
    0,
    N_("Player 2 type ('c' for computer, 'h' for human)"),
    N_("TYPE")
  },
  {
    NULL,
    '\0',
    0,
    NULL,
    0,
    NULL,
    NULL
  }
};

int
main (int argc, char *argv[])
{
  poptContext pctx;
  GtkWidget *app1;

#ifdef ENABLE_NLS
  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);
#endif

  gnome_init_with_popt_table (PACKAGE, VERSION, argc, argv,
                              options, 0, &pctx);
  poptFreeContext(pctx);

  if (width < 2)  g_error(_("Width must be >= 2."));
  if (height < 2)  g_error(_("Height must be >= 2."));
  if (width % 2 == 1)  g_error(_("Width must be an even number."));
  if (height % 2 == 1)  g_error(_("Height must be an even number."));

  /* init random numbers */
  srand(time(NULL));

  /* set game parameters */
  game.table = game_table_new (width, height);
  game.active_player = 0;

  if (p1 == NULL) game.player_cpu[0] = 0;
  else if (strcmp (p1, "h") == 0) game.player_cpu[0] = 0;
  else if (strcmp (p1, "c") == 0) game.player_cpu[0] = 1;
  else g_error(_("Player 1 type must be 'c' or 'h'"));;

  if (p2 == NULL) game.player_cpu[1] = 1;
  else if (strcmp (p2, "h") == 0) game.player_cpu[1] = 0;
  else if (strcmp (p2, "c") == 0) game.player_cpu[1] = 1;
  else g_error(_("Player 2 type must be 'c' or 'h'"));;

  /* create main window */
  app1 = create_app1 ();
  gtk_widget_show (app1);
  game.app1 = app1;

  /* draw the table */
  game_update_backup_xpm ();

  /* disable used buttons */
  gtk_widget_set_sensitive (game.button_color[game_table_get_color (game.table ,0)], 0);
  gtk_widget_set_sensitive (game.button_color[game_table_get_color (game.table ,1)], 0);

  /* write initial score */
  game_write_perc (0, game_table_get_number_of_rhombs(game.table, 0));
  game_write_perc (1, game_table_get_number_of_rhombs(game.table, 1));

  if (game.player_cpu[game.active_player]) game_computer_turn ();

  gtk_main ();

  game_table_destroy (game.table);

  return 0;
}

