/* sevencolors.h
 *
 * Copyright (C) 2000 Stefano Bellotti
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <gnome.h>

/* Corners of the game table */
typedef enum
{
  CORNER_LOW_LEFT  = 0,
  CORNER_TOP_RIGHT = 1
} GameTableCorner;

/* A rhomb */
typedef struct
{

  /* the color */
  int col;

  /* flag used by recursive calls */
  int flag;

} Rhomb;

/*
 * The gaming table
 * You should never modify the game table directly, use the supplied methods.
 */
typedef struct
{

  /* dimensions of the table (number of rhombs) */
  int width;
  int height;

  /* bidimensional array of rhombs */
  Rhomb **rhomb;

} GameTable;

/*
 * Creates a new allocated game table of the given size.
 * Rhombs colors are random, flags are unset.
 * The two corners always have different colors.
 */
GameTable *game_table_new (int width, int height);

/*
 * Relases all memory used by the game table.
 */
void game_table_destroy (GameTable *table);

/*
 * Randomizes the colors of a game table.
 * Rhombs colors are random, flags are unset.
 * The two corners always have different colors.
 */
void game_table_randomize (GameTable *table);

/*
 * Gets the color of the corner rhomb.
 */
int game_table_get_color (GameTable *table, GameTableCorner corner);

/*
 * Gets the number of rhombs owned by the player in that corner.
 */
int game_table_get_number_of_rhombs (GameTable *table, GameTableCorner corner);

/*
 * The color expands from the given corner.
 * Returns the total number of rhombs that get colored.
 */
int game_table_play_color (GameTable *table, int color, GameTableCorner corner);

/*
 * The ai chooses the best color.
 * Returns the choosen color.
 * TODO: implement a more intelligent ai.
 */
int game_table_computer_choose_color (GameTable *table, GameTableCorner corner);

/*
 * The ai chooses and plays the best color.
 * Returns the total number of rhombs that get colored and the choosen color.
 * TODO: implement a more intelligent ai.
 */
int game_table_computer_play (GameTable *table, GameTableCorner corner, int *color);

/*
 * Returns the total number of rhombs that would get colored if the player
 * in the given corner choses the given color.
 * The game table is not modified.
 */
int game_table_what_if (GameTable *table, GameTableCorner corner, int color);